/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.cache.ehcache.internal.rmi;

import com.liferay.portal.kernel.cache.SkipReplicationThreadLocal;

import java.io.Serializable;

import java.util.Collection;

import net.sf.ehcache.Ehcache;
import net.sf.ehcache.Element;
import net.sf.ehcache.constructs.EhcacheDecoratorAdapter;

/**
 * @author Tina Tian
 */
public class LiferayCacheDecorator extends EhcacheDecoratorAdapter {

	public LiferayCacheDecorator(Ehcache underlyingCache) {
		super(underlyingCache);
	}

	@Override
	public boolean equals(Object object) {
		return underlyingCache.equals(object);
	}

	public Ehcache getUnderlyingCache() {
		return underlyingCache;
	}

	@Override
	public int hashCode() {
		return underlyingCache.hashCode();
	}

	@Override
	public void put(Element element, boolean doNotNotifyCacheReplicators) {
		boolean enabled = SkipReplicationThreadLocal.isEnabled();

		SkipReplicationThreadLocal.setEnabled(doNotNotifyCacheReplicators);

		try {
			super.put(element, doNotNotifyCacheReplicators);
		}
		finally {
			SkipReplicationThreadLocal.setEnabled(enabled);
		}
	}

	@Override
	public Element putIfAbsent(
		Element element, boolean doNotNotifyCacheReplicators) {

		boolean enabled = SkipReplicationThreadLocal.isEnabled();

		SkipReplicationThreadLocal.setEnabled(doNotNotifyCacheReplicators);

		try {
			return super.putIfAbsent(element, doNotNotifyCacheReplicators);
		}
		finally {
			SkipReplicationThreadLocal.setEnabled(enabled);
		}
	}

	@Override
	public boolean remove(Object key, boolean doNotNotifyCacheReplicators) {
		boolean enabled = SkipReplicationThreadLocal.isEnabled();

		SkipReplicationThreadLocal.setEnabled(doNotNotifyCacheReplicators);

		try {
			return super.remove(key, doNotNotifyCacheReplicators);
		}
		finally {
			SkipReplicationThreadLocal.setEnabled(enabled);
		}
	}

	@Override
	public boolean remove(
		Serializable key, boolean doNotNotifyCacheReplicators) {

		return remove((Object)key, doNotNotifyCacheReplicators);
	}

	@Override
	public void removeAll(boolean doNotNotifyCacheReplicators) {
		boolean enabled = SkipReplicationThreadLocal.isEnabled();

		SkipReplicationThreadLocal.setEnabled(doNotNotifyCacheReplicators);

		try {
			super.removeAll(doNotNotifyCacheReplicators);
		}
		finally {
			SkipReplicationThreadLocal.setEnabled(enabled);
		}
	}

	@Override
	public void removeAll(
		Collection<?> keys, boolean doNotNotifyCacheReplicators) {

		boolean enabled = SkipReplicationThreadLocal.isEnabled();

		SkipReplicationThreadLocal.setEnabled(doNotNotifyCacheReplicators);

		try {
			super.removeAll(keys, doNotNotifyCacheReplicators);
		}
		finally {
			SkipReplicationThreadLocal.setEnabled(enabled);
		}
	}

}